library(tidyverse)
library(rstan)
library(lme4)
library(MuMIn)

##########################################################################################
# load data

# datasets loaded below are based on the CropPol field_level dataset from Allen-Perkins et al. 2022, available for download at https://esajournals.onlinelibrary.wiley.com/doi/10.1002/ecy.3614
# our version of this data has been processed as follows (note that more detail can be found in the paper methods section):
# 1) separated transect-level and field-level datasets to account for different random effects structure
# 2) minor bug fixes--these have been incorporated into most recent release of CropPol
# 3) created a study_id2 variable, which records the study each dataset is part of and does NOT separate different years as different studies as study_id does. There were four "studies" where the different years come from different papers, so we kept these separate (compare study_id to study_id2)
# 4) dropped data sets that didn't meet our criteria.  We dropped pan trap studies, enclosure studies, studies where honey bees were not recorded, studies without yield data, and a few studies with other data issues. 
# 5) some studies recorded pollinator visits, some recorded abundance, some have both. We used either visits or abundance depending on which one was available, with preference given to visits where both were available. This can be found in the "abund_or_visits_edited" column
# 6) we created a "taxa_recorded_edited" column that lists the taxa recorded for each dataset. If this was listed as NA in CropPol, we made an assumption based on the data or by finding the associated paper.
# 7) fixed management designation for one study which had listed management as unmanaged to organic (Christof_Schuepp_Prunus_avium_Switzerland_2011) and changed management designation from "NA" to "conventional" for ICP project datasets (field_data_S2) following conversation with Rufus Isaacs 2022-06-22. We also decided to subsume IPM to organic due to limited studies/fields and because its management paradigm is closer to organic than conventional
# 8) added climate, plant family, growth habit, region, pollinator reliance (based on Klein et al 2007 and other literature), and bloom time variables
# 9) cleaned up zero vs NA inconsistencies in the insect data 
# 10) removed 2 fields with insect or yield outliers that appeared to be due to recording errors
# 11) removed 10 fields where yields were 0 and don't represent normal situations for that crop system
# 12) created z-score versions of insect visits and crop yields


field_data_S1 = read.csv(file="dataset_S1.csv", header=T)	# 98 studies with field-level data; 1342 fields
field_data_S2 = read.csv(file="dataset_S2.csv", header=T)	# 22 studies with transect-level data; 194 fields, 741 transect measurements
landcover = read.csv(file="landcover.csv", header=T)		# landcover data, required for field-level analysis 


##########################################################################################
# Call bayesian analysis R script to run linear, segmented and flat models 
# there are two analysis parts:
# Analysis part 1, field-level data; separate stan scripts called:
# Analysis part 2, transect-level data; separate stan scripts called:
# produces a dataframe called lim_out

# set iterations for bayesian models
this_iterations = 1000 

# warning, this script will take a long time at 500k iterations as reported in the text!
#source("pollen_lim_bayesian_code.R")

# instead, bypass the above script and read in our results directly
lim_out = read.csv(file="lim_out.csv", header=T)

str(lim_out)

##########################################################################################
##########################################################################################
# RESULTS (1) How prevalent is pollinator limitation of crop yields worldwide?

#calculates those crop systems which may be pollinator limited because the linear or segmented model competes with the flate model (delta LOOIC < 2.00)
lim_out <-lim_out %>%
  mutate(delta2=case_when(decision=="0" & slope_lin >= 0& (LOOIC_flat-LOOIC_lin) <=0 & (LOOIC_flat-LOOIC_lin) >= -2 ~ "PFN-linear",
                          decision=="0" & slope_seg >= 0& (LOOIC_flat-LOOIC_seg) <=0 & (LOOIC_flat-LOOIC_seg) >= -2 ~ "PFN-segmented",
                          TRUE~"reg"))%>%
  mutate(perc_lim3 = case_when(delta2 == "PFN-segmented" ~ paste(perc_lim),
                               delta2 == "PFN-linear" ~ "1",
                               TRUE ~ paste(perc_lim2)))

lim_out$perc_lim3 <- as.numeric(lim_out$perc_lim3)
str(lim_out)

## lower estimate of pollinator limitation (does not consider competing models delta LOOIC < 2.00)
((sum(lim_out$perc_lim2 != 0)) /120)*100 # 28% of crop systems pollinator limited
sum(lim_out$perc_lim2 != 0) #33 crop systems that were pollinator limited; 87 which were not  
sum(lim_out$perc_lim2 == 1) #19 crop systems were fully limited (linear model) and 14 crop systems partially pollinator limited (33-19= 14) 

## higher estimate of pollinator limitation (includes competing models delta LOOIC < 2.00)
((sum(lim_out$perc_lim3 != 0)) /120)*100 # 61%
sum(lim_out$perc_lim3 != 0) #73 crop systems that were pollinator limited; 47 which were not 
sum(lim_out$perc_lim3 == 1) #40 crop systems were fully limited (linear model) and 33 crop systems partially pollinator limited (73-40= 33)

## number of crops
limcrops<-lim_out %>%
              dplyr::filter (perc_lim3 > 0) %>%
              group_by(crop)%>%
              dplyr::summarize(n_distinct(study))

#number of countries
limcountries<-lim_out %>%
              dplyr::filter (perc_lim3 > 0) %>%
              group_by(country)%>%
              dplyr::summarize(n_distinct(study))

##########################################################################################
##########################################################################################
# RESULTS (2) Are there risk factors that predict where pollinator limitation is more likely?

lim_out<-lim_out %>% select (-c(country, crop))
field_data_all = rbind(field_data_S1, field_data_S2)
field_data_all = merge(field_data_all, lim_out, by.x="study_id", by.y="study")

##########################################################################################
# Analysis of study-level factors

# We found no evidence for effects of climate, geographic region, plant family, growth habit, bloom phenology, or a crop’s expected reliance on pollinators on the occurrence of pollinator limitation within a study system. 

# create binary version of perc_lim2 that treats the segmented relationships as all 1's
field_data_all$perc_lim_binary = field_data_all$perc_lim2
field_data_all[field_data_all$perc_lim_binary >0,]$perc_lim_binary = 1

# summarize by study_id2 for study-level analysis
study_data_all <- (field_data_all %>%
                        group_by(study_id2) %>% 
                        dplyr::summarize(
                          perc_lim_binary = perc_lim_binary[1],
                          climate = climate[1],
                          crop = crop[1],
                          region = region[1],
                          type = type[1],
                          Family = Family[1],
                          reliance = reliance[1]
                        ))
study_data_all = data.frame(study_data_all)
head(study_data_all)

# making sure there are no NAs - this is very important for AIC selection since the datasets used in compared models must be the same!
study_data_all = subset(study_data_all, !is.na(study_data_all$reliance))
study_data_all = subset(study_data_all, !is.na(study_data_all$climate))
study_data_all = subset(study_data_all, !is.na(study_data_all$region))

study_data_all$region = as.factor(study_data_all$region)

# backwards selection using dredge()
# null model is best
mod1_study = glm(perc_lim_binary ~ reliance + region + climate + Family + crop, family= binomial, data=study_data_all, na.action="na.fail")
summary(mod1_study)
dredge(mod1_study) 

# Model selection table 
      # (Intrc) climt crop Famly regin relnc df   logLik   AICc   delta weight
# 1  -1.082e+00                               1  -44.697   91.4    0.00  0.667
# 2  -1.946e+00     +                         3  -44.216   94.8    3.31  0.128
# 17 -1.872e+00                            +  4  -43.241   95.0    3.58  0.112
# 9  -6.931e-01                      +        6  -41.679   96.5    5.08  0.053
# 18 -2.718e+00     +                      +  6  -42.760   98.7    7.24  0.018
#...

#############################################
# Analysis of study-level bloom phenology 

# define spring blooming crops
spring<-c("Allium porrum","Brassica napus","Brassica rapa","Fragaria x ananassa","Malus domestica","Prunus avium",
          "Prunus cerasus","Prunus dulcis","Pyrus communis","Vaccinium corymbosum", "Vicia faba")

# create subset of studies relevant to phenology analysis
phenologydata <- study_data_all %>% 
  filter(climate=="Temperate")%>%
  filter(!crop %in% c("Actinidia chinensis","Actinidia deliciosa"))%>% #remove subtropical kiwi vines
  mutate(bloom=case_when(crop %in% spring ~ "Spring",
                         !crop %in% spring ~ "Other"))
phenologydata$bloom<-as.factor(phenologydata$bloom)

# backwards selection using dredge()
# null competing as best model

mod1_bloom = glm(perc_lim_binary ~ bloom + reliance + region + Family + type, family = binomial, data=phenologydata, na.action="na.fail")
summary(mod1_bloom)
dredge(mod1_bloom) 

#Model selection table 
#       (Intrc) bloom Famly regin relnc type df  logLik AICc delta weight
# 1  -9.985e-01                               1 -30.290 62.7  0.00  0.274
# 5   1.657e+01                 +             4 -27.657 64.2  1.51  0.129
# 2  -5.596e-01     +                         2 -29.987 64.2  1.56  0.125
# 10 -1.685e+00     +                 +       4 -27.834 64.5  1.86  0.108
# 18  2.274e-01     +                      +  5 -26.779 64.9  2.20  0.091
# 9  -1.946e+00                       +       3 -29.475 65.5  2.79  0.068
# 17 -1.041e+00                            +  4 -28.846 66.5  3.88  0.039
# 6   1.657e+01     +           +             5 -27.625 66.6  3.89  0.039
# 3  -5.220e-15           +                   8 -23.873 67.1  4.44  0.030
# 4   1.850e+01     +     +                   9 -22.774 67.8  5.17  0.021
# 13  1.541e+01                 +     +       6 -26.994 67.9  5.19  0.020
#...

##########################################################################################
# Analysis of field-level factors

# determine whether each field is limited by comparing to the breakpoint value for all_insects_z
# for the logistic regression, we create a binary "is limited" variable that is 0/1 for each field as follows:

# using the lower estimate of pollinator limitation
field_data_all$is_limited = field_data_all$perc_lim2

for (i in 1:nrow(field_data_all)) {
  
  if (field_data_all[i,]$perc_lim2 > 0 & field_data_all[i,]$perc_lim2 < 1) {
    
    if(field_data_all[i,]$all_insects_z >= field_data_all[i,]$breakpt) { field_data_all[i,]$is_limited <- 0 }
    if(field_data_all[i,]$all_insects_z < field_data_all[i,]$breakpt) { field_data_all[i,]$is_limited <- 1 }
  }
}

# next we needed to combine this with landcover data
# processing step: reduce Allium fields to representative cultivar C only, because all cultivars had same lat/longs
field_data_logistic<-field_data_all %>%
  filter(!study_id %in%
           c("Thijs_Fijen_Allium_porrum_Italy_2016_B", "Thijs_Fijen_Allium_porrum_Italy_2016_D", "Thijs_Fijen_Allium_porrum_Italy_2016_E", "Thijs_Fijen_Allium_porrum_Italy_2016_F", "Thijs_Fijen_Allium_porrum_France_2016_B", "Thijs_Fijen_Allium_porrum_France_2016_D", "Thijs_Fijen_Allium_porrum_France_2016_E", "Thijs_Fijen_Allium_porrum_France_2016_F"))
field_data_logistic$study_id[field_data_logistic$study_id == 'Thijs_Fijen_Allium_porrum_Italy_2016_C'] <- 'Thijs_Fijen_Allium_porrum_Italy_2016'
field_data_logistic$study_id[field_data_logistic$study_id == 'Thijs_Fijen_Allium_porrum_France_2016_C'] <- 'Thijs_Fijen_Allium_porrum_France_2016'

#merge landcover composition dataset with the main dataset
field_data_logistic<-field_data_logistic%>%
  mutate(study_site= paste(study_id, site_id, sep = "_"))

field_data_logistic = merge(field_data_logistic,landcover,by="study_site") #1388 of 1415 fields had usable coordinates
length(unique(field_data_logistic$study_id2))	#79

# landcover processing for analysis
field_data_logistic<-field_data_logistic %>%
  mutate(openhabitat_1k= grass_1000 + crop_1000)%>%
  mutate(woodyhabitat_1k=shrub_1000 + tree_1000)%>%
  mutate(openhabitat_4k= grass_4000 + crop_4000)%>%
  mutate(woodyhabitat_4k=shrub_4000 + tree_4000)

# creating scaled versions of landcover variables
field_data_logistic$urban.cs.1k<-scale(field_data_logistic$urban_1000)
field_data_logistic$shrub.cs.1k<-scale(field_data_logistic$shrub_1000)
field_data_logistic$tree.cs.1k<-scale(field_data_logistic$tree_1000)
field_data_logistic$crop.cs.1k<-scale(field_data_logistic$crop_1000)
field_data_logistic$grass.cs.1k<-scale(field_data_logistic$grass_1000)
field_data_logistic$urban.cs.4k<-scale(field_data_logistic$urban_4000)
field_data_logistic$shrub.cs.4k<-scale(field_data_logistic$shrub_4000)
field_data_logistic$tree.cs.4k<-scale(field_data_logistic$tree_4000)
field_data_logistic$crop.cs.4k<-scale(field_data_logistic$crop_4000)
field_data_logistic$grass.cs.4k<-scale(field_data_logistic$grass_4000)
field_data_logistic$open.cs.1k<-scale(field_data_logistic$openhabitat_1k)
field_data_logistic$woody.cs.1k<-scale(field_data_logistic$woodyhabitat_1k)
field_data_logistic$open.cs.4k<-scale(field_data_logistic$openhabitat_4k)
field_data_logistic$woody.cs.4k<-scale(field_data_logistic$woodyhabitat_4k)
field_data_logistic$evergreen.cs.4k<-scale(field_data_logistic$evergreen_4000)
field_data_logistic$evergreen.cs.1k<-scale(field_data_logistic$evergreen_1000)
field_data_logistic$decid.cs.4k<-scale(field_data_logistic$decid_4000)
field_data_logistic$decid.cs.1k<-scale(field_data_logistic$decid_1000)
field_data_logistic$rshrub.cs.4k<-scale(field_data_logistic$rshrub_4000)
field_data_logistic$rshrub.cs.1k<-scale(field_data_logistic$rshrub_1000)

#############################################
# Logistic regression analysis of field-level datasets

# null model
mod_null = glmer(is_limited~(1|study_id2), family=binomial, data=field_data_logistic, na.action="na.fail",control=glmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
AICc(mod_null)

# forward model selection resulted in two competing single-variable models

mod1_tree1k <- glmer(is_limited~tree.cs.1k + (1|study_id2), family=binomial, data=field_data_logistic, na.action="na.fail",control=glmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
AICc(mod1_tree1k)

mod2_tree4k <- glmer(is_limited~tree.cs.4k + (1|study_id2), family=binomial, data=field_data_logistic, na.action="na.fail",control=glmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
AICc(mod2_tree4k) 

# forward model selection: at the next step, one two-variable competed with the single-variable models

mod3_tree1k_tree4k<- glmer(is_limited~tree.cs.1k+tree.cs.4k+(1|study_id2), family=binomial, data=field_data_logistic, na.action="na.fail",control=glmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
AICc(mod3_tree1k_tree4k) 

# however, the significance of both predictors decreases from single models and 4k is non significant, evidence to not select fuller model (sutherland et al. 2023;https://doi.org/10.1098/rspb.2023.1261)

# model selection table
#                      K   AICc Delta_AICc AICcWt Cum.Wt      LL
# mod1_tree.1k         3 537.02       0.00   0.39   0.39 -265.50 
# mod3_tree4k+tree1k   4 537.08       0.06   0.38   0.76 -264.52 
# mod2_tree.4k         3 538.02       1.00   0.23   1.00 -266.00
# mod_null             2 547.01       9.99   0.00   1.00 -271.50

# thus, we chose the following as the best model:
bestmodel <- glmer(is_limited~tree.cs.1k + (1|study_id2), family=binomial, data=field_data_logistic, na.action="na.fail",control=glmerControl(optimizer="bobyqa",optCtrl=list(maxfun=2e5)))
summary(bestmodel) 

#############################################
# examining context dependency in the effect of forest cover

mean_tree_vals = mean(aggregate(data=field_data_logistic, tree.cs.1k~study_id2, FUN=mean)$V1)
forest_q10 = quantile(field_data_logistic$tree.cs.1k,.1)
forest_q90 = quantile(field_data_logistic$tree.cs.1k,.9)

# extract random effects and calculate CIs
a=fixef(bestmodel)
a_overall = a[1]			# overall intercept
a_tree.cs.1k = a[2]			# overall slope of forest
b=ranef(bestmodel)$study_id2[,1]	# random effect intercepts
revars <- attr(ranef(bestmodel, condVar = TRUE)[[1]], "postVar") # variances of the random effects
e = (sqrt(revars))[,,] # standard errors of random effects

mean_farms_forest_q10 = b + a_overall+ a_tree.cs.1k*forest_q10 	
mean_farms_forest_q90 = b + a_overall+ a_tree.cs.1k*forest_q90 

out = data.frame(cbind(mean_farms_forest_q10, mean_farms_forest_q90))
invLogit <- function(x) exp(x)/(1 + exp(x))
out = invLogit(out)

# calculate change in probability of limitation with an increase of surrounding forested landcover from the 10th to the 90th percentile of observed values 
out<-out%>%
  mutate(forest.diff= mean_farms_forest_q90-mean_farms_forest_q10)%>%
  mutate(study= paste(unique(field_data_logistic$study_id2)))
 
arrange(out, forest.diff)


##########################################################################################
##########################################################################################
# RESULTS (3) By how much could increases in pollinator visitation improve crop yields?

gapsum<-lim_out %>%
  filter (perc_lim2 > 0)
  
median(gapsum$gap_perc) # 63%
mean(gapsum$gap_perc) # 64%


#############################################
# example yield gap plot
# Raspberry in Argentina

thisstudy = subset(field_data_all, study_id=="Agustin_Saez_Rubus_idaeus_Argentina_2014")

# set up bayesian model (linear version was best model for this crop system)
print(this_iterations)

stanDat <- list(
	yield_z = thisstudy$yield_z, 
	all_insects_z = thisstudy$all_insects_z,
	N = nrow(thisstudy)
)

mod_linear <- stan(file = "pollen lim linear non-mixed.stan", data = stanDat, iter = this_iterations, chains = 4)
summary(mod_linear)

xvals1 = seq(-5,5,.01)
beta2 = as.data.frame(mod_linear)["beta[2]"][[1]]
beta1 = as.data.frame(mod_linear)["beta[1]"][[1]]
out = data.frame(x = xvals1, mean = 0, lowerCI = 0, upperCI=0)
for (i in 1:length(xvals1)) {
	y = beta2*out$x[i] + beta1
	out$mean[i] = mean(y)
	out$lowerCI[i] = quantile(y, .025)
	out$upperCI[i] = quantile(y, .975)
}

# back-transform to original crop units from z-scores
this_pollinator_q10 = quantile(thisstudy$all_insects, .1)
this_pollinator_q90 = quantile(thisstudy$all_insects, .9)
this_yield_q10 = quantile(thisstudy$yield, .1)
this_yield_q90 = quantile(thisstudy$yield, .9)
sd_all_insects = sd(thisstudy$all_insects)
sd_yield = sd(thisstudy$yield)
mean_all_insects = mean(thisstudy$all_insects)
mean_yield = mean(thisstudy$yield)

slope_lin_untrans = summary(mod_linear)$summary[2,1]*(sd_yield/sd_all_insects)
intercept_lin_untrans = summary(mod_linear)$summary[1,1]*sd_yield + mean_yield - slope_lin_untrans*mean_all_insects

this_yield_gap_q10_intersection = slope_lin_untrans*this_pollinator_q10 + intercept_lin_untrans
this_yield_gap_q90_intersection = slope_lin_untrans*this_pollinator_q90 + intercept_lin_untrans

out$x_ut = out$x*sd_all_insects + mean_all_insects 
out$lowerCI_ut = out$lowerCI*sd_yield + mean_yield 
out$upperCI_ut = out$upperCI*sd_yield + mean_yield 
xvals1_ut = seq(-5*sd(thisstudy$all_insects)+mean(thisstudy$all_insects),5*sd(thisstudy$all_insects)+mean(thisstudy$all_insects),.01)

# plot yield gap figure
plot(thisstudy$all_insects, thisstudy$yield, ylab="Crop Yield", xlab="Insect Visitation", ylim=c(0,3*sd(thisstudy$yield)+mean(thisstudy$yield)), xlim=c(0,3*sd(thisstudy$all_insects)+mean(thisstudy$all_insects)), cex=1.5, cex.main=1.5, cex.lab=1.25, main="Raspberry in Argentina")
polygon(c(out$x_ut, rev(out$x_ut)), c(out$upperCI_ut, rev(out$lowerCI_ut)), col = "#fce184", border=F)
abline(v = this_pollinator_q10, lty="dotted", lwd=2)
abline(v = this_pollinator_q90, lty="dotted", lwd=2)
abline(h = this_yield_q10, lty="solid", col="#4792b1", lwd=4)
abline(h = this_yield_q90, lty="solid", col="#4792b1", lwd=4)
points(xvals1_ut, (intercept_lin_untrans + slope_lin_untrans*(xvals1_ut)), col="black", type="l", lwd=2)
abline(h = this_yield_gap_q10_intersection, lty="dashed", col="#ba0100", lwd=4)
abline(h = this_yield_gap_q90_intersection, lty="dashed", col="#ba0100", lwd=4)
points(thisstudy$all_insects, thisstudy$yield, pch=19, cex=1.5, col="#4f4f4f")
